%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CalcCC.m 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Calculates Re(C) and Im(C) for a given frequency and given Wt and Ws
%       weights corresponding to that frequency which are for a specific
%       plant that is G=Gr+i*Gi at that frequency
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   inputs :
%       ths     : Gridded thetaS (vector)
%       Ws      : Sensitivity Weight for frequency 'f' (scalar)
%       Wt      : Complementary Sensitivity for frequency 'f' (scalar)
%       Gr      : Real part of the plant for frequency 'f' (scalar)
%       Gi      : Imaginary part of the plant for frequency 'f' (scalar)
%
%   outputs :
%       Kr      : Real part of the controller (scalar)
%       Ki      : Imaginary part of the controller (scalar)
%       solL    : absolute value for the loop gains if any solution exists
%       (vector)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Kr, Ki, solL] = CalcCC(thS,Ws,Wt,Gr,Gi)

% calculation of discriminant, |L| values
% note that both must be positive for existance of a solution
cthS=cos(thS);                          %calculation of the cotangent of thetaS values
mLdisc=(Wt*(Ws-cthS)).^2-(1-Wt^2)*(2*Ws*cthS-Ws^2-1);           %calculation of the discriminant
signD=sign(mLdisc);                     %sign of determinants
smLa=(Wt*(Ws-cthS)+sqrt(mLdisc))/(1-Wt^2);      %calculation of loop gains
smLb=(Wt*(Ws-cthS)-sqrt(mLdisc))/(1-Wt^2);      %calculation of loop gains

% check the solution for existance
solL =[];
solthS = [];
jreg = 1;
%disp(num2str(signD))
for i = 1:length(signD)
    if (signD(i) >= 0)
        if ((imag(smLa(i))==0) & (real(smLa(i))>=0))
            solL(jreg) = smLa(i);
            solthS(jreg) = thS(i);
            jreg = jreg + 1;
        end;
    end
end

for i = 1:length(signD)
    if (signD(i) >= 0)
        if ((imag(smLb(i))==0) & (real(smLb(i))>=0))
            solL(jreg) = smLb(i);
            solthS(jreg) = thS(i);
            jreg = jreg + 1;
        end;
    end;
end;

% Calculate Kr=Re(C) and Ki=Im(C)
if (length(solL)==0)        %if no solition exists 
    Kr = [];
    Ki = [];
else                        %if some solution exists
    % calculation of Re(L), Im(L) values
    Lr=(Ws+Wt*solL).*cos(solthS)-1;
    Li=-(Ws+Wt*solL).*sin(solthS);
    % calculation of Re(C), Im(C) values
    Kr=(Gr*Lr+Gi*Li)/(Gr^2+Gi^2);
    Ki=(Gr*Li-Gi*Lr)/(Gr^2+Gi^2);
end;